:root {
    --icon-font-family: "Material Icons";
}

@font-face {
    font-family: "Material Icons";
    font-style: normal;
    font-weight: 400;
    src: url("../font/materialicons/v143/MaterialIcons-Regular.woff2") format("woff2");
}

@media all {
    .material-icons {
        direction: ltr;
        display: inline-block;
        font-family: var(--icon-font-family);
        font-size: 24px; /* Preferred icon size */
        font-style: normal;
        font-weight: normal;
        letter-spacing: normal;
        line-height: 1;
        overflow-wrap: normal;
        text-transform: none;
        white-space: nowrap;
    }
}

@media all {
    .aa-sovtimer-timer-elapsed {
        color: var(--bs-danger);
    }

    .aa-sov-timer-zkb-icon img {
        position: relative;
        top: -2px;
    }

    .aa-sovtimer-active-campaign {
        border-left: 1px solid var(--bs-danger);
        border-right: 1px solid var(--bs-danger);
    }

    .aa-sovtimer-active-campaign td {
        background-color: var(--bs-danger-bg-subtle) !important;

        --bs-table-color-type: var(--bs-danger-text-emphasis) !important;
    }

    .aa-sovtimer-upcoming-campaign {
        border-left: 1px solid var(--bs-warning);
        border-right: 1px solid var(--bs-warning);
    }

    .aa-sovtimer-upcoming-campaign td {
        background-color: var(--bs-warning-bg-subtle) !important;

        --bs-table-color-type: var(--bs-warning-text-emphasis) !important;
    }

    .aa-sovtimer-trend {
        cursor: default;
        display: inline-block;
        left: 0.35rem;
        margin-right: 0.75rem;
        position: relative;
        top: 0.35rem;
    }

    .aa-sovtimer-trend-up {
        color: var(--bs-success);
    }

    .aa-sovtimer-trend-down {
        color: var(--bs-danger);
    }

    .aa-sovtimer .aa-sovtimer-campaigns td {
        vertical-align: middle;
    }
}

/* Loading Spinner
------------------------------------------------------------------------------------- */
@media all {
    .aa-sovtimer .loading-spinner {
        width: 2.5rem;
        height: 2.5rem;
    }
}

/* DataTables
------------------------------------------------------------------------------------- */
@media all {
    .dt-container .dt-info {
        margin-top: 0.85rem;
    }

    /* DataTables ColumnControl
    --------------------------------------------------------------------------------- */
    table.dataTable span.dtcc div.dtcc-search>div select {
        background-color: var(--bs-body-bg);
        color: var(--bs-body-color);
        padding: 0.375rem;
    }

    table.dataTable .dt-column-header div.dtcc-search-title {
        display: none;
    }
}
