# Generated by Django 3.1.3 on 2020-11-13 10:33

# Django
import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("eveuniverse", "0004_effect_longer_name"),
        ("sovtimer", "0002_aasovtimercampaigns_aasovtimerstructures"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="aasovtimercampaigns",
            name="constellation_id",
        ),
        migrations.RemoveField(
            model_name="aasovtimercampaigns",
            name="defender_id",
        ),
        migrations.RemoveField(
            model_name="aasovtimercampaigns",
            name="solar_system_id",
        ),
        migrations.RemoveField(
            model_name="aasovtimerstructures",
            name="alliance_id",
        ),
        migrations.RemoveField(
            model_name="aasovtimerstructures",
            name="solar_system_id",
        ),
        migrations.AddField(
            model_name="aasovtimercampaigns",
            name="defender",
            field=models.ForeignKey(
                blank=True,
                default=None,
                null=True,
                on_delete=django.db.models.deletion.SET_DEFAULT,
                related_name="sov_campaign_defender",
                to="eveuniverse.eveentity",
            ),
        ),
        migrations.AddField(
            model_name="aasovtimercampaigns",
            name="solar_system",
            field=models.ForeignKey(
                blank=True,
                default=None,
                null=True,
                on_delete=django.db.models.deletion.SET_DEFAULT,
                related_name="sov_capaign_solar_system",
                to="eveuniverse.evesolarsystem",
            ),
        ),
        migrations.AddField(
            model_name="aasovtimerstructures",
            name="alliance",
            field=models.ForeignKey(
                blank=True,
                default=None,
                null=True,
                on_delete=django.db.models.deletion.SET_DEFAULT,
                related_name="sov_structure_alliance",
                to="eveuniverse.eveentity",
            ),
        ),
        migrations.AddField(
            model_name="aasovtimerstructures",
            name="solar_system",
            field=models.ForeignKey(
                blank=True,
                default=None,
                null=True,
                on_delete=django.db.models.deletion.SET_DEFAULT,
                related_name="sov_structure_solar_system",
                to="eveuniverse.evesolarsystem",
            ),
        ),
    ]
