# Generated by Django 4.0.10 on 2023-09-22 22:46

# Django
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("sovtimer", "0008_fix_campaign_attackers_score"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="campaign",
            options={
                "default_permissions": (),
                "verbose_name": "Sovereignty campaign",
                "verbose_name_plural": "Sovereignty campaigns",
            },
        ),
        migrations.AlterModelOptions(
            name="sovereigntystructure",
            options={
                "default_permissions": (),
                "verbose_name": "Sovereignty structure",
                "verbose_name_plural": "Sovereignty structures",
            },
        ),
        migrations.AlterField(
            model_name="campaign",
            name="event_type",
            field=models.CharField(
                choices=[
                    ("ihub_defense", "IHub defense"),
                    ("tcu_defense", "TCU defense"),
                ],
                max_length=12,
            ),
        ),
    ]
