# Generated by Django 3.1.3 on 2020-11-13 18:56

# Django
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("sovtimer", "0003_auto_20201113_1033"),
    ]

    operations = [
        migrations.AddField(
            model_name="aasovtimercampaigns",
            name="progress_current",
            field=models.FloatField(default=0.6),
        ),
        migrations.AddField(
            model_name="aasovtimercampaigns",
            name="progress_previous",
            field=models.FloatField(default=0.6),
        ),
        migrations.AlterField(
            model_name="aasovtimercampaigns",
            name="attackers_score",
            field=models.FloatField(default=0.6),
        ),
        migrations.AlterField(
            model_name="aasovtimercampaigns",
            name="defender_score",
            field=models.FloatField(default=0.6),
        ),
    ]
