# Generated by Django 3.2.11 on 2022-01-30 11:36

# Django
import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("eveuniverse", "0005_type_materials_and_sections"),
        ("sovtimer", "0006_rename_models"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="campaign",
            name="defender",
        ),
        migrations.RemoveField(
            model_name="campaign",
            name="solar_system",
        ),
        migrations.RemoveField(
            model_name="campaign",
            name="structure_id",
        ),
        migrations.AddField(
            model_name="campaign",
            name="structure",
            field=models.OneToOneField(
                blank=True,
                default=None,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="sov_capaign_structure",
                to="sovtimer.sovereigntystructure",
            ),
        ),
        migrations.AlterField(
            model_name="sovereigntystructure",
            name="alliance",
            field=models.ForeignKey(
                blank=True,
                default=None,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="sov_structure_alliance",
                to="eveuniverse.eveentity",
            ),
        ),
        migrations.AlterField(
            model_name="sovereigntystructure",
            name="solar_system",
            field=models.ForeignKey(
                blank=True,
                default=None,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="sov_structure_solar_system",
                to="eveuniverse.evesolarsystem",
            ),
        ),
        migrations.AlterField(
            model_name="sovereigntystructure",
            name="vulnerability_occupancy_level",
            field=models.FloatField(default=1),
        ),
        migrations.AlterField(
            model_name="sovereigntystructure",
            name="vulnerable_end_time",
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name="sovereigntystructure",
            name="vulnerable_start_time",
            field=models.DateTimeField(blank=True, null=True),
        ),
    ]
